// Copyright (C) 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#pragma once

#include <sys/mman.h>

#include <Device.h>
#include <Logging.h>

class TranscoderDevice : public Device {
private:
  auto GetBarInfo(unsigned Index) {
    pci_dev pciDev = GetPciDevice();
    return std::make_pair(pciDev.base_addr[Index] & ~0xf, pciDev.size[Index] & ~0xf);
  }

  void* GetVirtAddr(uint64_t Length, uint64_t Offset) {
    auto vaddr = mmap(NULL, Length, PROT_READ | PROT_WRITE, MAP_SHARED, m_device, Offset);
    if (vaddr == MAP_FAILED) {
      logFatal("mmap failed\n");
      fail();
    }
    return vaddr;
  }

public:
  explicit TranscoderDevice(const pci_dev& PciDevice, bool Demo);
  void InitTransDevice();
  void TestTransDevice();
  void TestReset();

  ~TranscoderDevice() {
    if (!IsDemo() && GetPciDevice().device_id == 0x5070) {
#if 0
      TestReset();
#endif
      close(m_device);
      munmap(m_bar2Vaddr, m_bar2Size);
      munmap(m_bar4Vaddr, m_bar4Size);
    }
  }
};
